@@ -566,7 +566,7 @@ class SemanticTokensClientCapabilitiesRequestsFull(Model):
 
 @dataclass(repr=False)
 class SemanticTokensClientCapabilitiesRequests(Model):
-    range: Union[bool, Dict[Any, Any], None]
+    range: Union[bool, Dict[Any, Any], None] = None
     full: Union[SemanticTokensClientCapabilitiesRequestsFull, bool, None] = None
 
 
@@ -1045,7 +1045,7 @@ class LinkedEditingRangeOptions(WorkDoneProgressOptions):
 
 @dataclass(repr=False)
 class LinkedEditingRangeRegistrationOptions(
-    TextDocumentRegistrationOptions, LinkedEditingRangeOptions, StaticRegistrationOptions
+    LinkedEditingRangeOptions, StaticRegistrationOptions, TextDocumentRegistrationOptions
 ):
     pass
 
@@ -1068,7 +1068,7 @@ class RenameOptions(WorkDoneProgressOptions):
 
 
 @dataclass(repr=False)
-class RenameRegistrationOptions(TextDocumentRegistrationOptions, RenameOptions, StaticRegistrationOptions):
+class RenameRegistrationOptions(RenameOptions, StaticRegistrationOptions, TextDocumentRegistrationOptions):
     pass
 
 
@@ -1078,7 +1078,24 @@ class InlineValueOptions(WorkDoneProgressOptions):
 
 
 @dataclass(repr=False)
-class InlineValueRegistrationOptions(InlineValueOptions, TextDocumentRegistrationOptions, StaticRegistrationOptions):
+class InlineValueRegistrationOptions(TextDocumentRegistrationOptions, StaticRegistrationOptions, InlineValueOptions):
+    pass
+
+
+@dataclass(repr=False)
+class _DiagnosticOptions(Model):
+    inter_file_dependencies: bool
+    workspace_diagnostics: bool
+    identifier: Optional[str] = None
+
+
+@dataclass(repr=False)
+class DiagnosticOptions(WorkDoneProgressOptions, _DiagnosticOptions):
+    pass
+
+
+@dataclass(repr=False)
+class DiagnosticRegistrationOptions(TextDocumentRegistrationOptions, StaticRegistrationOptions, DiagnosticOptions):
     pass
 
 
@@ -1115,6 +1132,9 @@ class ServerCapabilities(Model):
 
     # TODO typeHierarchyProvider?: boolean | TypeHierarchyOptions | TypeHierarchyRegistrationOptions;
     inline_value_provider: Union[bool, InlineValueOptions, InlineValueRegistrationOptions, None] = None
+    # TODO inlayHintProvider?: boolean | InlayHintOptions | InlayHintRegistrationOptions;
+    diagnostic_provider: Union[DiagnosticOptions, DiagnosticRegistrationOptions, None] = None
+
     workspace_symbol_provider: Union[bool, WorkspaceSymbolOptions, None] = None
     workspace: Optional[ServerCapabilitiesWorkspace] = None
     experimental: Optional[Any] = None
@@ -1203,6 +1223,9 @@ class Position(Model):
 
         return range.start <= self < range.end
 
+    def __hash__(self) -> int:
+        return hash((self.line, self.character))
+
 
 @dataclass(repr=False)
 class Range(Model):
@@ -1225,6 +1248,19 @@ class Range(Model):
             ),
         )
 
+    @staticmethod
+    def invalid() -> "Range":
+        return Range(
+            start=Position(
+                line=-1,
+                character=-1,
+            ),
+            end=Position(
+                line=-1,
+                character=-1,
+            ),
+        )
+
     def extend(self, start_line: int = 0, start_character: int = 0, end_line: int = 0, end_character: int = 0) -> Range:
         return Range(
             start=Position(line=self.start.line + start_line, character=self.start.character + start_character),
@@ -1239,6 +1275,9 @@ class Range(Model):
     def is_in_range(self, range: Range) -> bool:
         return range.start.is_in_range(self) and range.end.is_in_range(self)
 
+    def __hash__(self) -> int:
+        return hash((self.start, self.end))
+
 
 @dataclass(repr=False)
 class TextDocumentItem(Model):
@@ -1397,6 +1436,9 @@ class Location(Model):
     uri: DocumentUri
     range: Range
 
+    def __hash__(self) -> int:
+        return hash((self.uri, self.range))
+
 
 @dataclass(repr=False)
 class LocationLink(Model):
@@ -2160,3 +2202,118 @@ class InlineValueEvaluatableExpression(Model):
 
 
 InlineValue = Union[InlineValueText, InlineValueVariableLookup, InlineValueEvaluatableExpression]
+
+
+@dataclass(repr=False)
+class _DocumentDiagnosticParams(Model):
+    text_document: TextDocumentIdentifier
+    identifier: Optional[str] = None
+    previous_result_id: Optional[str] = None
+
+
+@dataclass(repr=False)
+class DocumentDiagnosticParams(WorkDoneProgressParams, PartialResultParams, _DocumentDiagnosticParams):
+    pass
+
+
+class DocumentDiagnosticReportKind(Enum):
+    FULL = "full"
+    UNCHANGED = "unchanged"
+
+
+@dataclass(repr=False)
+class FullDocumentDiagnosticReport(Model):
+    items: List[Diagnostic]
+    result_id: Optional[str] = None
+    kind: Literal[DocumentDiagnosticReportKind.FULL] = DocumentDiagnosticReportKind.FULL
+
+
+@dataclass(repr=False)
+class UnchangedDocumentDiagnosticReport(Model):
+    result_id: str
+    kind: DocumentDiagnosticReportKind = DocumentDiagnosticReportKind.UNCHANGED
+
+
+@dataclass(repr=False)
+class RelatedFullDocumentDiagnosticReport(FullDocumentDiagnosticReport):
+    related_documents: Optional[
+        Dict[DocumentUri, Union[FullDocumentDiagnosticReport, UnchangedDocumentDiagnosticReport]]
+    ] = None
+
+
+@dataclass(repr=False)
+class RelatedUnchangedDocumentDiagnosticReport(UnchangedDocumentDiagnosticReport):
+    related_documents: Optional[
+        Dict[DocumentUri, Union[FullDocumentDiagnosticReport, UnchangedDocumentDiagnosticReport]]
+    ] = None
+
+
+DocumentDiagnosticReport = Union[RelatedFullDocumentDiagnosticReport, RelatedUnchangedDocumentDiagnosticReport]
+
+
+@dataclass(repr=False)
+class DocumentDiagnosticReportPartialResult(Model):
+    related_documents: Optional[
+        Dict[DocumentUri, Union[FullDocumentDiagnosticReport, UnchangedDocumentDiagnosticReport]]
+    ] = None
+
+
+@dataclass(repr=False)
+class DiagnosticServerCancellationData(Model):
+    retrigger_request: bool
+
+
+@dataclass(repr=False)
+class PreviousResultId(Model):
+    uri: DocumentUri
+    value: str
+
+
+@dataclass(repr=False)
+class _WorkspaceDiagnosticParams(Model):
+    previous_result_ids: List[PreviousResultId]
+    identifier: Optional[str] = None
+
+
+@dataclass(repr=False)
+class WorkspaceDiagnosticParams(WorkDoneProgressParams, PartialResultParams, _WorkspaceDiagnosticParams):
+    pass
+
+
+@dataclass(repr=False)
+class _WorkspaceFullDocumentDiagnosticReport(Model):
+    uri: DocumentUri
+    version: Optional[int]
+
+
+@dataclass(repr=False)
+class WorkspaceFullDocumentDiagnosticReport(FullDocumentDiagnosticReport, _WorkspaceFullDocumentDiagnosticReport):
+    pass
+
+
+@dataclass(repr=False)
+class _WorkspaceUnchangedDocumentDiagnosticReport(Model):
+    uri: DocumentUri
+    version: Optional[int]
+
+
+@dataclass(repr=False)
+class WorkspaceUnchangedDocumentDiagnosticReport(
+    UnchangedDocumentDiagnosticReport, _WorkspaceUnchangedDocumentDiagnosticReport
+):
+    pass
+
+
+WorkspaceDocumentDiagnosticReport = Union[
+    WorkspaceFullDocumentDiagnosticReport, WorkspaceUnchangedDocumentDiagnosticReport
+]
+
+
+@dataclass(repr=False)
+class WorkspaceDiagnosticReportPartialResult(Model):
+    items: List[WorkspaceDocumentDiagnosticReport]
+
+
+@dataclass(repr=False)
+class WorkspaceDiagnosticReport(Model):
+    items: List[WorkspaceDocumentDiagnosticReport]
