@@ -16,6 +16,7 @@ from ...utils.async_tools import async_event
 from ...utils.logging import LoggingDescriptor
 from .has_extend_capabilities import HasExtendCapabilities
 from .lsp_types import (
+    UTF16,
     CancelParams,
     ClientCapabilities,
     ClientInfo,
@@ -49,6 +50,7 @@ from .parts.folding_range import FoldingRangeProtocolPart
 from .parts.formatting import FormattingProtocolPart
 from .parts.hover import HoverProtocolPart
 from .parts.implementation import ImplementationProtocolPart
+from .parts.inline_value import InlineValueProtocolPart
 from .parts.linked_editing_ranges import LinkedEditingRangeProtocolPart
 from .parts.references import ReferencesProtocolPart
 from .parts.rename import RenameProtocolPart
@@ -88,6 +90,7 @@ class LanguageServerProtocol(JsonRPCProtocol):
     linked_editing_range = ProtocolPartDescriptor(LinkedEditingRangeProtocolPart)
     selection_range = ProtocolPartDescriptor(SelectionRangeProtocolPart)
     rename = ProtocolPartDescriptor(RenameProtocolPart)
+    inline_value = ProtocolPartDescriptor(InlineValueProtocolPart)
 
     name: Optional[str] = None
     short_name: Optional[str] = None
@@ -191,6 +194,13 @@ class LanguageServerProtocol(JsonRPCProtocol):
         try:
             self.initialization_options = initialization_options
             try:
+                if (
+                    self.client_capabilities.general
+                    and self.client_capabilities.general.position_encodings
+                    and UTF16 in self.client_capabilities.general.position_encodings
+                ):
+                    self.capabilities.position_encoding = UTF16
+
                 await self.on_initialize(self, initialization_options)
             except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
                 raise
