@@ -39,7 +39,6 @@ class TextDocument:
         super().__init__()
 
         self._lock = Lock()
-        self._references: weakref.WeakSet[Any] = weakref.WeakSet()
         self.document_uri = document_uri
         self.uri = Uri(self.document_uri).normalized()
         self.language_id = language_id
@@ -52,14 +51,14 @@ class TextDocument:
         self._data: weakref.WeakKeyDictionary[Any, Any] = weakref.WeakKeyDictionary()
         self.opened_in_editor = False
 
-    @property
-    def references(self) -> weakref.WeakSet[Any]:  # pragma: no cover
-        return self._references
-
     @property
     def version(self) -> Optional[int]:
         return self._version
 
+    @version.setter
+    def version(self, value: Optional[int]) -> None:
+        self._version = value
+
     def __str__(self) -> str:  # pragma: no cover
         return self.__repr__()
 
@@ -145,8 +144,10 @@ class TextDocument:
         return self._lines
 
     async def get_lines(self) -> List[str]:
-        async with self._lock:
-            return self.__get_lines()
+        if self._lines is None:
+            async with self._lock:
+                return self.__get_lines()
+        return self._lines
 
     def _invalidate_cache(self) -> None:
         self._cache.clear()
