@@ -30,9 +30,16 @@ from typing import (
 
 from ....utils.path import path_is_relative_to
 from ...common.lsp_types import Position, Range
+from ..utils.ast_utils import Token, range_from_token
 from ..utils.markdownformatter import MarkDownFormatter
+from ..utils.match import normalize
 from ..utils.version import get_robot_version
-from .entities import ImportedVariableDefinition, VariableDefinition
+from .entities import (
+    ArgumentDefinition,
+    ImportedVariableDefinition,
+    SourceEntity,
+    VariableDefinition,
+)
 
 RUN_KEYWORD_NAMES = [
     "Run Keyword",
@@ -105,10 +112,9 @@ class KeywordMatcher:
 
     @property
     def normalized_name(self) -> str:
-        from robot.utils.normalizing import normalize
 
         if self._normalized_name is None:
-            self._normalized_name = str(normalize(self.name, "_"))
+            self._normalized_name = str(normalize(self.name))
 
         return self._normalized_name
 
@@ -126,8 +132,6 @@ class KeywordMatcher:
         return self._embedded_arguments
 
     def __eq__(self, o: Any) -> bool:
-        from robot.utils.normalizing import normalize
-
         if isinstance(o, KeywordMatcher):
             if not self.embedded_arguments:
                 return self.normalized_name == o.normalized_name
@@ -140,7 +144,7 @@ class KeywordMatcher:
         if self.embedded_arguments:
             return self.embedded_arguments.name.match(o) is not None
 
-        return self.normalized_name == str(normalize(o, "_"))
+        return self.normalized_name == str(normalize(o))
 
     def __hash__(self) -> int:
         return hash(
@@ -284,32 +288,37 @@ class ArgumentSpec:
 
 
 @dataclass
-class KeywordDoc:
+class KeywordDoc(SourceEntity):
     name: str = ""
-    args: Tuple[KeywordArgumentDoc, ...] = ()
-    doc: str = ""
+    name_token: Optional[Token] = field(default=None, compare=False)
+    args: Tuple[KeywordArgumentDoc, ...] = field(default=(), compare=False)
+    doc: str = field(default="", compare=False)
     tags: Tuple[str, ...] = ()
-    source: Optional[str] = None
-    line_no: int = -1
-    end_line_no: int = -1
     type: str = "keyword"
     libname: Optional[str] = None
     libtype: Optional[str] = None
     longname: Optional[str] = None
     is_embedded: bool = False
-    errors: Optional[List[Error]] = None
+    errors: Optional[List[Error]] = field(default=None, compare=False)
     doc_format: str = DEFAULT_DOC_FORMAT
     is_error_handler: bool = False
-    error_handler_message: Optional[str] = None
+    error_handler_message: Optional[str] = field(default=None, compare=False)
     is_initializer: bool = False
-    is_registered_run_keyword: bool = False
-    args_to_process: Optional[int] = None
-    deprecated: bool = False
-    arguments: Optional[ArgumentSpec] = None
+    is_registered_run_keyword: bool = field(default=False, compare=False)
+    args_to_process: Optional[int] = field(default=None, compare=False)
+    deprecated: bool = field(default=False, compare=False)
+    arguments: Optional[ArgumentSpec] = field(default=None, compare=False)
+    argument_definitions: Optional[List[ArgumentDefinition]] = field(default=None, compare=False)
 
     def __str__(self) -> str:
         return f"{self.name}({', '.join(str(arg) for arg in self.args)})"
 
+    @property
+    def matcher(self) -> KeywordMatcher:
+        if not hasattr(self, "__matcher"):
+            self.__matcher = KeywordMatcher(self.name)
+        return self.__matcher
+
     @property
     def is_deprecated(self) -> bool:
         return self.deprecated or DEPRECATED_PATTERN.match(self.doc) is not None
@@ -328,15 +337,32 @@ class KeywordDoc:
             return m.group("message").strip()
         return ""
 
+    @property
+    def name_range(self) -> Range:
+        if self.name_token is not None:
+            return range_from_token(self.name_token)
+        else:
+            return Range.invalid()
+
     @property
     def range(self) -> Range:
-        return Range(
-            start=Position(line=self.line_no - 1 if self.line_no >= 0 else 0, character=0),
-            end=Position(
-                line=self.end_line_no - 1 if self.end_line_no >= 0 else self.line_no if self.line_no >= 0 else 0,
-                character=0,
-            ),
-        )
+        if self.name_token is not None:
+            return range_from_token(self.name_token)
+        else:
+            return Range(
+                start=Position(
+                    line=self.line_no - 1 if self.line_no > 0 else 0,
+                    character=self.col_offset if self.col_offset >= 0 else 0,
+                ),
+                end=Position(
+                    line=self.end_line_no - 1 if self.end_line_no >= 0 else self.line_no if self.line_no > 0 else 0,
+                    character=self.end_col_offset
+                    if self.end_col_offset >= 0
+                    else self.col_offset
+                    if self.col_offset >= 0
+                    else 0,
+                ),
+            )
 
     def to_markdown(self, add_signature: bool = True, header_level: int = 0) -> str:
         if self.doc_format == DEFAULT_DOC_FORMAT:
@@ -410,7 +436,26 @@ class KeywordDoc:
         return self.libname == BUILTIN_LIBRARY_NAME and self.name == RUN_KEYWORDS_NAME
 
     def __hash__(self) -> int:
-        return id(self)
+        # return id(self)
+        return hash(
+            (
+                self.name,
+                self.longname,
+                self.source,
+                self.line_no,
+                self.col_offset,
+                self.end_line_no,
+                self.end_col_offset,
+                self.type,
+                self.libname,
+                self.libtype,
+                self.is_embedded,
+                self.is_initializer,
+                self.is_error_handler,
+                self.doc_format,
+                self.tags,
+            )
+        )
 
 
 class KeywordError(Exception):
@@ -502,7 +547,7 @@ class LibraryDoc:
     source: Optional[str] = None
     line_no: int = -1
     end_line_no: int = -1
-    inits: KeywordStore = field(default_factory=KeywordStore)
+    inits: KeywordStore = field(default_factory=KeywordStore, compare=False)
     keywords: KeywordStore = field(default_factory=KeywordStore, compare=False)
     module_spec: Optional[ModuleSpec] = None
     errors: Optional[List[Error]] = field(default=None, compare=False)
@@ -1227,9 +1272,10 @@ def get_library_doc(
                             tags=tuple(kw[0].tags),
                             source=kw[0].source,
                             line_no=kw[0].lineno,
+                            col_offset=-1,
+                            end_col_offset=-1,
+                            end_line_no=-1,
                             type="library",
-                            # libname=kw[1].libname,
-                            # longname=kw[1].longname,
                             libname=libdoc.name,
                             libtype=libdoc.type,
                             longname=f"{libdoc.name}.{kw[0].name}",
@@ -1268,6 +1314,9 @@ def get_library_doc(
                             tags=tuple(kw[0].tags),
                             source=kw[0].source,
                             line_no=kw[0].lineno,
+                            col_offset=-1,
+                            end_col_offset=-1,
+                            end_line_no=-1,
                             libname=libdoc.name,
                             libtype=libdoc.type,
                             longname=f"{libdoc.name}.{kw[0].name}",
