@@ -7,12 +7,14 @@ from typing import TYPE_CHECKING, Any, List, Optional
 from ....utils.async_tools import check_canceled, threaded
 from ....utils.logging import LoggingDescriptor
 from ...common.decorators import language_id
-from ...common.lsp_types import Diagnostic, DiagnosticSeverity, Position, Range
-from ...common.parts.diagnostics import (
-    DOCUMENT_DIAGNOSTICS_DEBOUNCE,
-    WORKSPACE_DIAGNOSTICS_DEBOUNCE,
-    DiagnosticsResult,
+from ...common.lsp_types import (
+    Diagnostic,
+    DiagnosticSeverity,
+    DiagnosticTag,
+    Position,
+    Range,
 )
+from ...common.parts.diagnostics import DiagnosticsResult
 from ...common.text_document import TextDocument
 from ..diagnostics.analyzer import Analyzer
 from ..utils.ast_utils import (
@@ -41,18 +43,23 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
 
         parent.diagnostics.collect.add(self.collect_namespace_diagnostics)
 
+        parent.diagnostics.collect.add(self.collect_unused_references)
+
         parent.documents_cache.namespace_invalidated.add(self.namespace_invalidated)
 
     @language_id("robotframework")
     async def namespace_invalidated(self, sender: Any, document: TextDocument) -> None:
-        await self.parent.diagnostics.create_publish_document_diagnostics_task(
-            document,
-            wait_time=DOCUMENT_DIAGNOSTICS_DEBOUNCE if document.opened_in_editor else WORKSPACE_DIAGNOSTICS_DEBOUNCE,
-        )
+        self.parent.diagnostics.cancel_workspace_diagnostics()
+        self.parent.diagnostics.cancel_document_diagnostics(document)
+        await self.parent.diagnostics.set_collect_full_diagnostics(False)
 
     @language_id("robotframework")
     @threaded()
-    async def collect_namespace_diagnostics(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+    @_logger.call
+    async def collect_namespace_diagnostics(self, sender: Any, document: TextDocument, full: bool) -> DiagnosticsResult:
+        return await document.get_cache(self._collect_namespace_diagnostics, full)
+
+    async def _collect_namespace_diagnostics(self, document: TextDocument, full: bool) -> DiagnosticsResult:
         try:
             namespace = await self.parent.documents_cache.get_namespace(document)
             if namespace is None:
@@ -116,8 +123,12 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
         )
 
     @language_id("robotframework")
+    @threaded()
     @_logger.call
-    async def collect_token_errors(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+    async def collect_token_errors(self, sender: Any, document: TextDocument, full: bool) -> DiagnosticsResult:
+        return await document.get_cache(self._collect_token_errors, full)
+
+    async def _collect_token_errors(self, document: TextDocument, full: bool) -> DiagnosticsResult:
         from robot.errors import VariableError
         from robot.parsing.lexer.tokens import Token
 
@@ -183,7 +194,10 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
     @language_id("robotframework")
     @threaded()
     @_logger.call
-    async def collect_model_errors(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+    async def collect_model_errors(self, sender: Any, document: TextDocument, full: bool) -> DiagnosticsResult:
+        return await document.get_cache(self._collect_model_errors, full)
+
+    async def _collect_model_errors(self, document: TextDocument, full: bool) -> DiagnosticsResult:
 
         from ..utils.ast_utils import HasError, HasErrors
         from ..utils.async_ast import iter_nodes
@@ -228,3 +242,75 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                     )
                 ],
             )
+
+    @language_id("robotframework")
+    @threaded()
+    @_logger.call
+    async def collect_unused_references(self, sender: Any, document: TextDocument, full: bool) -> DiagnosticsResult:
+        if not full:
+            return DiagnosticsResult(self.collect_unused_references, [])
+        return await document.get_cache(self._collect_unused_references, full)
+
+    async def _collect_unused_references(self, document: TextDocument, full: bool) -> DiagnosticsResult:
+        if not full:
+            return DiagnosticsResult(self.collect_unused_references, [])
+
+        try:
+            namespace = await self.parent.documents_cache.get_namespace(document)
+            if namespace is None:
+                return DiagnosticsResult(self.collect_unused_references, None)
+
+            result: List[Diagnostic] = []
+            for kw in (await namespace.get_library_doc()).keywords.values():
+                references = await self.parent.robot_references.find_keyword_references(document, kw, False)
+                if not references and not await Analyzer.should_ignore(document, kw.name_range):
+                    result.append(
+                        Diagnostic(
+                            range=kw.name_range,
+                            message=f"Keyword '{kw.name}' is not used.",
+                            severity=DiagnosticSeverity.WARNING,
+                            source=self.source_name,
+                            code="KeywordNotUsed",
+                            tags=[DiagnosticTag.Unnecessary],
+                        )
+                    )
+
+            for var in await namespace.get_own_variables():
+                references = await self.parent.robot_references.find_variable_references(document, var, False)
+                if not references and not await Analyzer.should_ignore(document, var.name_range):
+                    result.append(
+                        Diagnostic(
+                            range=var.name_range,
+                            message=f"Variable '{var.name}' is not used.",
+                            severity=DiagnosticSeverity.WARNING,
+                            source=self.source_name,
+                            code="VariableNotUsed",
+                            tags=[DiagnosticTag.Unnecessary],
+                        )
+                    )
+
+            return DiagnosticsResult(self.collect_unused_references, result)
+        except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
+            raise
+        except BaseException as e:
+            return DiagnosticsResult(
+                self.collect_unused_references,
+                [
+                    Diagnostic(
+                        range=Range(
+                            start=Position(
+                                line=0,
+                                character=0,
+                            ),
+                            end=Position(
+                                line=len(await document.get_lines()),
+                                character=len((await document.get_lines())[-1] or ""),
+                            ),
+                        ),
+                        message=f"Fatal: can't collect unused references '{e}' ({type(e).__qualname__})",
+                        severity=DiagnosticSeverity.ERROR,
+                        source=self.source_name,
+                        code=type(e).__qualname__,
+                    )
+                ],
+            )
