@@ -3,13 +3,17 @@ from __future__ import annotations
 import asyncio
 from typing import TYPE_CHECKING, Any, List, Optional
 
-from ....utils.async_tools import threaded
+from ....utils.async_tools import Event, threaded
 from ....utils.glob_path import iter_files
 from ....utils.logging import LoggingDescriptor
 from ....utils.uri import Uri
 from ...common.decorators import language_id
-from ...common.parts.diagnostics import DiagnosticsMode, WorkspaceDocumentsResult
-from ..configuration import AnalysisConfig, AnalysisProgressMode, RobotCodeConfig
+from ...common.parts.diagnostics import (
+    AnalysisProgressMode,
+    DiagnosticsMode,
+    WorkspaceDocumentsResult,
+)
+from ..configuration import AnalysisConfig, RobotCodeConfig
 from ..diagnostics.library_doc import RESOURCE_FILE_EXTENSION, ROBOT_FILE_EXTENSION
 
 if TYPE_CHECKING:
@@ -28,9 +32,10 @@ class RobotWorkspaceProtocolPart(RobotLanguageServerProtocolPart):
     def __init__(self, parent: RobotLanguageServerProtocol) -> None:
         super().__init__(parent)
         self.parent.documents.on_read_document_text.add(self._on_read_document_text)
-        self.parent.diagnostics.collect_workspace_documents.add(self._collect_workspace_documents)
+        self.parent.diagnostics.load_workspace_documents.add(self._load_workspace_documents)
         self.parent.diagnostics.on_get_diagnostics_mode.add(self.on_get_diagnostics_mode)
-        self.workspace_loaded = False
+        self.parent.diagnostics.on_get_analysis_progress_mode.add(self.on_get_analysis_progress_mode)
+        self.workspace_loaded = Event()
 
     @language_id("robotframework")
     async def _on_read_document_text(self, sender: Any, uri: Uri) -> Optional[str]:
@@ -43,8 +48,12 @@ class RobotWorkspaceProtocolPart(RobotLanguageServerProtocolPart):
         config = await self.parent.workspace.get_configuration(AnalysisConfig, uri)
         return config.diagnostic_mode
 
+    async def on_get_analysis_progress_mode(self, sender: Any, uri: Uri) -> Optional[AnalysisProgressMode]:
+        config = await self.parent.workspace.get_configuration(AnalysisConfig, uri)
+        return config.progress_mode
+
     @threaded()
-    async def _collect_workspace_documents(self, sender: Any) -> List[WorkspaceDocumentsResult]:
+    async def _load_workspace_documents(self, sender: Any) -> List[WorkspaceDocumentsResult]:
 
         result: List[WorkspaceDocumentsResult] = []
 
@@ -101,10 +110,7 @@ class RobotWorkspaceProtocolPart(RobotLanguageServerProtocolPart):
                     except BaseException as e:
                         self._logger.exception(e)
 
-        self.workspace_loaded = True
-
-        if config.analysis.references_code_lens:
-            await self.parent.code_lens.refresh()
+        self.workspace_loaded.set()
 
         if canceled:
             return []
