@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import concurrent.futures
+import contextlib
 import contextvars
 import functools
 import inspect
@@ -200,6 +201,7 @@ class AsyncTaskingEventResultIteratorBase(AsyncEventResultIteratorBase[_TCallabl
         result_callback: Optional[Callable[[Optional[_TResult], Optional[BaseException]], Any]] = None,
         return_exceptions: Optional[bool] = True,
         callback_filter: Optional[Callable[[_TCallable], bool]] = None,
+        threaded: Optional[bool] = True,
         **kwargs: Any,
     ) -> AsyncIterator[Union[_TResult, BaseException]]:
         def _done(f: asyncio.Future[_TResult]) -> None:
@@ -217,7 +219,7 @@ class AsyncTaskingEventResultIteratorBase(AsyncEventResultIteratorBase[_TCallabl
             set(self),
         ):
             if method is not None:
-                if isinstance(method, HasThreaded) and cast(HasThreaded, method).__threaded__:
+                if threaded and isinstance(method, HasThreaded) and cast(HasThreaded, method).__threaded__:
                     future = run_coroutine_in_thread(ensure_coroutine(method), *args, **kwargs)
                 else:
                     future = create_sub_task(ensure_coroutine(method)(*args, **kwargs))
@@ -434,6 +436,9 @@ def check_canceled_sync() -> bool:
     return True
 
 
+__threadpool_executor = ThreadPoolExecutor(thread_name_prefix="sub_asyncio")
+
+
 def run_in_thread(func: Callable[..., _T], /, *args: Any, **kwargs: Any) -> asyncio.Future[_T]:
     global __tread_pool_executor
 
@@ -444,7 +449,7 @@ def run_in_thread(func: Callable[..., _T], /, *args: Any, **kwargs: Any) -> asyn
 
     return cast(
         "asyncio.Future[_T]",
-        loop.run_in_executor(None, cast(Callable[..., _T], func_call)),
+        loop.run_in_executor(__threadpool_executor, cast(Callable[..., _T], func_call)),
     )
 
 
@@ -475,11 +480,26 @@ def run_coroutine_in_thread(
                 inner_task.cancel()
 
             return await inner_task
+
         finally:
             threading.current_thread().setName(old_name)
 
     def run() -> _T:
-        return asyncio.run(create_inner_task())
+        loop = asyncio.new_event_loop()
+        try:
+            asyncio.set_event_loop(loop)
+
+            return loop.run_until_complete(create_inner_task())
+        finally:
+            try:
+                running_tasks = asyncio.all_tasks(loop)
+                if running_tasks:
+                    loop.run_until_complete(asyncio.gather(*running_tasks, return_exceptions=True))
+
+                loop.run_until_complete(loop.shutdown_asyncgens())
+            finally:
+                asyncio.set_event_loop(None)
+                loop.close()
 
     cti = get_current_future_info()
     result = run_in_thread(run)
@@ -503,12 +523,28 @@ def run_coroutine_in_thread(
     return result
 
 
+_pool = concurrent.futures.ThreadPoolExecutor(max_workers=1)
+
+
+@contextlib.asynccontextmanager
+async def async_lock(lock: threading.RLock) -> AsyncGenerator[None, None]:
+    locked = lock.acquire(False)
+    while not locked:
+        await asyncio.sleep(0)
+        locked = lock.acquire(False)
+    try:
+        yield
+    finally:
+        if locked:
+            lock.release()
+
+
 class Event:
     """Thread safe version of an async Event"""
 
-    def __init__(self) -> None:
+    def __init__(self, value: bool = False) -> None:
         self._waiters: Deque[asyncio.Future[Any]] = deque()
-        self._value = False
+        self._value = value
         self._lock = threading.RLock()
 
     def __repr__(self) -> str:
@@ -519,39 +555,184 @@ class Event:
         return f"<{res[1:-1]} [{extra}]>"
 
     def is_set(self) -> bool:
-        return self._value
+        with self._lock:
+            return self._value
 
     def set(self) -> None:
-        if not self._value:
-            self._value = True
+        with self._lock:
+            if not self._value:
+                self._value = True
+
+                while self._waiters:
+                    fut = self._waiters.popleft()
 
-            with self._lock:
-                for fut in self._waiters:
                     if not fut.done():
                         if fut._loop == asyncio.get_running_loop():
-                            fut.set_result(True)
+                            if not fut.done():
+                                fut.set_result(True)
                         else:
-                            fut._loop.call_soon_threadsafe(fut.set_result, True)
+
+                            def s(w: asyncio.Future[Any], ev: threading.Event) -> None:
+                                try:
+                                    if not w.done():
+                                        w.set_result(True)
+                                finally:
+                                    ev.set()
+
+                            if not fut.done():
+                                done = threading.Event()
+
+                                fut._loop.call_soon_threadsafe(s, fut, done)
+
+                                if not done.wait(120):
+                                    raise RuntimeError("Callback timeout")
 
     def clear(self) -> None:
-        self._value = False
-
-    async def wait(self) -> bool:
-        if self._value:
-            return True
-
         with self._lock:
+            self._value = False
+
+    async def wait(self, timeout: Optional[float] = None) -> bool:
+        with self._lock:
+            if self._value:
+                return True
+
             fut = create_sub_future()
             self._waiters.append(fut)
+
         try:
-            await fut
+            await asyncio.wait_for(fut, timeout)
             return True
-        finally:
-            with self._lock:
-                self._waiters.remove(fut)
+        except asyncio.TimeoutError:
+            return False
+
+
+class Semaphore:
+    """Thread safe version of a Semaphore"""
+
+    def __init__(self, value: int = 1) -> None:
+        if value < 0:
+            raise ValueError("Semaphore initial value must be >= 0")
+        self._value = value
+        self._waiters: Deque[asyncio.Future[Any]] = deque()
+
+        self._lock = threading.RLock()
+
+    def __repr__(self) -> str:
+        res = super().__repr__()
+        extra = "locked" if self.locked() else f"unlocked, value:{self._value}"
+        if self._waiters:
+            extra = f"{extra}, waiters:{len(self._waiters)}"
+        return f"<{res[1:-1]} [{extra}]>"
+
+    async def _wake_up_next(self) -> None:
+        async with async_lock(self._lock):
+            while self._waiters:
+                waiter = self._waiters.popleft()
+
+                if not waiter.done():
+                    if waiter._loop == asyncio.get_running_loop():
+                        if not waiter.done():
+                            waiter.set_result(True)
+                    else:
+                        if waiter._loop.is_running():
+
+                            def s(w: asyncio.Future[Any], ev: threading.Event) -> None:
+                                try:
+                                    if w._loop.is_running() and not w.done():
+                                        w.set_result(True)
+                                finally:
+                                    ev.set()
+
+                            if not waiter.done():
+                                done = threading.Event()
+
+                                waiter._loop.call_soon_threadsafe(s, waiter, done)
+
+                                if not done.wait(120):
+                                    raise RuntimeError("Callback timeout")
+
+                        else:
+                            raise RuntimeError("Loop is not running.")
+
+    def locked(self) -> bool:
+        with self._lock:
+            return self._value == 0
+
+    async def acquire(self, timeout: Optional[float] = None) -> bool:
+        while self._value <= 0:
+            fut = create_sub_future()
+            async with async_lock(self._lock):
+                self._waiters.append(fut)
+            try:
+                await asyncio.wait_for(fut, timeout)
+            except asyncio.TimeoutError:
+                return False
+            except BaseException:
+                if not fut.done():
+                    fut.cancel()
+                if self._value > 0 and not fut.cancelled():
+                    await self._wake_up_next()
+
+                raise
+
+        async with async_lock(self._lock):
+            self._value -= 1
+
+        return True
+
+    async def release(self) -> None:
+        self._value += 1
+        await self._wake_up_next()
+
+    async def __aenter__(self) -> None:
+        await self.acquire()
+
+    async def __aexit__(
+        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
+    ) -> None:
+        await self.release()
+
+
+class BoundedSemaphore(Semaphore):
+    """Thread safe version of a BoundedSemaphore"""
+
+    def __init__(self, value: int = 1) -> None:
+        self._bound_value = value
+        super().__init__(value)
+
+    async def release(self) -> None:
+        if self._value >= self._bound_value:
+            raise ValueError("BoundedSemaphore released too many times")
+        await super().release()
 
 
 class Lock:
+    def __init__(self) -> None:
+        self._block = BoundedSemaphore(value=1)
+
+    def __repr__(self) -> str:
+        return "<%s _block=%s>" % (self.__class__.__name__, self._block)
+
+    async def acquire(self, timeout: Optional[float] = None) -> bool:
+        return await self._block.acquire(timeout)
+
+    async def release(self) -> None:
+        await self._block.release()
+
+    @property
+    def locked(self) -> bool:
+        return self._block.locked()
+
+    async def __aenter__(self) -> None:
+        await self.acquire()
+
+    async def __aexit__(
+        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
+    ) -> None:
+        await self.release()
+
+
+class OldLock:
     """Threadsafe version of an async Lock."""
 
     def __init__(self) -> None:
@@ -585,6 +766,12 @@ class Lock:
             yield None
         finally:
             self._lock.release()
+        # with self._lock:
+        #     yield None
+
+    @property
+    def locked(self) -> bool:
+        return self._locked
 
     async def acquire(self) -> bool:
         async with self.__inner_lock():
@@ -603,33 +790,36 @@ class Lock:
                 await fut
             finally:
                 async with self.__inner_lock():
-                    self._waiters.remove(fut)
+                    if fut in self._waiters:
+                        self._waiters.remove(fut)
                     self._locked = True
         except asyncio.CancelledError:
-            async with self.__inner_lock():
-                if self._locked:
-                    await self._wake_up_first()
+            await self._wake_up_next()
             raise
 
         return True
 
     async def release(self) -> None:
         async with self.__inner_lock():
-            if self._locked:
-                self._locked = False
-            else:
-                raise RuntimeError("Lock is not acquired.")
+            if self._waiters is None or len(self._waiters) == 0:
+                if self._locked:
+                    self._locked = False
+                else:
+                    raise RuntimeError(f"Lock is not acquired ({len(self._waiters) if self._waiters else 0} waiters).")
 
-            await self._wake_up_first()
+        await self._wake_up_next()
 
-    async def _wake_up_first(self) -> None:
+    async def _wake_up_next(self) -> None:
         if not self._waiters:
             return
 
-        try:
-            fut = next(iter(self._waiters))
-        except StopIteration:
-            return
+        async with self.__inner_lock():
+            try:
+                fut = next(iter(self._waiters))
+            except StopIteration:
+                return
+            if fut in self._waiters:
+                self._waiters.remove(fut)
 
         def s() -> None:
             if not fut.done():
@@ -639,7 +829,8 @@ class Lock:
             if not fut.done():
                 fut.set_result(True)
         else:
-            fut._loop.call_soon_threadsafe(s)
+            if fut._loop.is_running():
+                fut._loop.call_soon_threadsafe(s)
 
 
 class FutureInfo:
