@@ -40,7 +40,7 @@ def start_debugpy(port: int, wait_for_client: bool) -> bool:
 
         real_port = check_free_port(port)
         if real_port != port:
-            _logger.info(f"start debugpy session on port {real_port}")
+            _logger.warning(f"start debugpy session on port {real_port}")
         debugpy.listen(real_port)
 
         if wait_for_client:
